#version 330 core

struct Light {
    vec3 position;
    vec3 color;
    float linear;
    float quadratic;
    float radius;
};


in vec2 _texUV;
smooth in vec3 _normal;

in vec3 _eyePos;
in vec4 _fragPosLightSpace;
in vec3 _fragPos;
in vec3 _pointlightPos1;
in vec3 _pointlightPos2;
in vec3 _lightPos;

in vec3 tmpT;
in vec3 tmpBT;

uniform sampler2D color_texture;
uniform sampler2D shadow_texture;
uniform sampler2D bumpmap_texture;

uniform vec4 color;
uniform vec4 ambientColor;
uniform vec4 specularColor;

uniform float shininess;
uniform bool hasColorTexture;

uniform vec3 lightDir;

uniform vec3 eyePos;

uniform Light pointLight1;
uniform Light pointLight2;

uniform float bloomHighpassFilterFactor = 0.90f;

layout(location = 0) out vec4 fragColor;

uniform bool hasBumpMapTexture = true;

#define AMBIENT_LIGHT 0.1f


vec3 calculateBumpMappingNormal(vec3 normal)
{
	// Create Tangent Space
	vec3 up = vec3(0,1,0);
	vec3 tangent = normalize(cross(up, normal));
	if(length(tangent) == 0)
		tangent = vec3(1, 0, 0);
	vec3 bitangent = normalize(cross(normal, tangent));

	mat3 tbnMatrix = mat3(tangent, bitangent, normal);
	vec3 bumpNormal = normalize(texture(bumpmap_texture, _texUV).xyz * 2.0 - 1.0);

	return normalize(tbnMatrix * bumpNormal);
}

float specularComponent(vec3 lightDirection, vec3 viewDirection, vec3 normal, float shininess)
{
	if(dot(normal, lightDirection) < 0.0)
		return 0.0;

	return pow(max(0.0, dot(reflect(-lightDirection, normal), viewDirection)), shininess);
}

float calculateShadowValue(vec4 fragPosLightSpace)
{
	//calculate the positions between [-1,1]
	vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
	//calculate the positions between [0,1]
	projCoords = projCoords * 0.5 + 0.5; 

	float clostestDepthLight = texture(shadow_texture, projCoords.xy).r;
	//float clostestDepthLight = shadow2DProj(shadow_texture, fragPosLightSpace).r;
	float currentDepth = projCoords.z;  


	float bias = 0.003; //0.003
//float bias = max(0.05 * (1.0 - dot(normalViewspace, lightDir)), 0.005);  

	//float shadow = currentDepth - bias > clostestDepthLight  ? 1.0 : 0.0;

	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadow_texture, 0);
	for(int x = -1; x <= 1; ++x)
	{
    		for(int y = -1; y <= 1; ++y)
    		{
        		float pcfDepth = texture(shadow_texture, projCoords.xy + vec2(x, y) * texelSize).r; 
        		shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;        
    		}    
	}
	shadow /= 9.0;	

	if(projCoords.z > 1.0)
        	shadow = 0.0;

	return shadow;
}

void main()
{
	vec3 n;
	if(hasBumpMapTexture == true)
	{
		n = normalize(texture(bumpmap_texture, _texUV).xyz * 2.0 - 1.0);
	}else{
		n = normalize( _normal );
	}


	vec3 lightDirection = _lightPos - _fragPos;
	vec3 lightDirectionNorm = normalize(lightDirection);
	vec3 viewDirection = normalize(_eyePos - _fragPos);
 
	vec4 texColor = (hasColorTexture) ? texture(color_texture, _texUV) : color;
	float alphavalue = texColor.a;

	vec4 ambient = ambientColor;
 
	vec4 diffuse = vec4(color.xyz * clamp(dot(lightDirectionNorm, n), 0.0, 1.0),1);

	vec3 spec = specularColor.xyz * specularComponent(lightDirectionNorm, viewDirection, n, shininess) * 0.8f;


//-.-.-.--.-..-.-.-.-.-pointlight1-.-.--.-.-.--.-.-.-.-.-.-.-.-
	lightDirection = _pointlightPos1 - _fragPos;
	lightDirectionNorm = normalize(lightDirection);

	float distance = length(lightDirection);
	if(distance <= pointLight1.radius)
	{
		float attenuation = 1.0 / (1.0 + pointLight1.linear * distance + pointLight1.quadratic * distance * distance);

		diffuse += vec4(pointLight1.color * clamp(dot(lightDirectionNorm, n), 0.0, 1.0) * attenuation,1);
		spec += pointLight1.color * specularComponent(lightDirectionNorm, viewDirection, n, shininess) * 0.8f * attenuation;
	}
//-.-.-.--.-..-.-.-.-.-pointlight2-.-.--.-.-.--.-.-.-.-.-.-.-.-

	lightDirection = _pointlightPos2 - _fragPos;
	lightDirectionNorm = normalize(lightDirection);

	distance = length(lightDirection);
	if(distance <= pointLight2.radius)
	{
		float attenuation = 1.0 / (1.0 + pointLight2.linear * distance + pointLight2.quadratic * distance * distance);

		diffuse += vec4(pointLight2.color * clamp(dot(lightDirectionNorm, n), 0.0, 1.0) * attenuation,1);
		spec += pointLight2.color * specularComponent(lightDirectionNorm, viewDirection, n, shininess) * 0.8f * attenuation;
	}
//-.-.-.-.-.-.-.-.-.-.--.-.--.-..-.-.-.-.-..-.-..--.-.-..-.-.-


fragColor = (ambient + diffuse) * texColor + vec4( spec, 1 );


float shadow = calculateShadowValue(_fragPosLightSpace);
fragColor *= (1-shadow);
fragColor.a = alphavalue;
//if(hasBumpMapTexture == true)
//{
//	fragColor.xyz = tmpT;
//	fragColor.a = 1.0f;
//}
}